//==============================================================================
//	File:		EthernetLoopback (code snippet)
//	Author:		Chris Fletcher (http://cwfletcher.net)
//	Copyright:	Copyright 2010 UC Berkeley
//==============================================================================

//==============================================================================
//	Section:	License
//==============================================================================
//	Copyright (c) 2010, Regents of the University of California
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification,
//	are permitted provided that the following conditions are met:
//
//		- Redistributions of source code must retain the above copyright notice,
//			this list of conditions and the following disclaimer.
//		- Redistributions in binary form must reproduce the above copyright
//			notice, this list of conditions and the following disclaimer
//			in the documentation and/or other materials provided with the
//			distribution.
//		- Neither the name of the University of California, Berkeley nor the
//			names of its contributors may be used to endorse or promote
//			products derived from this software without specific prior
//			written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//==============================================================================

//------------------------------------------------------------------------------
//	Module:		EthernetLoopback
//	Desc:		This file contains a code snippet that will help you verify your 
//				Ethernet/CPU Adapter.  To use this code, follow these steps:
//					1.) Copy this code into your FPGA_TOP_ML505/UART testbench.
//					2.) Create a means of sending a packet (contents are up to 
//						you) through the transmit side of your Ethernet/CPU 
//						Adapter.
//				This module will loop-back whatever comes out on the TX side to 
//				the RX side.  Thus, it will show you how EMAC asserts signals in 
//				accepting the packet you created (from CPU->EMAC).  After the 
//				packet is transmitted, the EMAC on the RX side will receive it 
//				verbatim (this is what is meant by loopback) and pass it through 
//				your RX path.  This will give you an idea of how the EMAC 
//				asserts signals on the RX side.
//	Author:		<a href="http://cwfletcher.net">Chris Fletcher</a>
//------------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Parameters
	//--------------------------------------------------------------------------
	parameter		ClockFreq =						100_000_000,		
					EWidth =						8,
					LoopDelay =						1,
					TXClockDelay =					2,
					RXClockDelay =					3,
					Index =							-1,					
					HalfCycle = 					5;							//Half of the Clock Period is 10 ns
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Wires & Regs
	//--------------------------------------------------------------------------
	wire					PHY_COL, PHY_CRS;
	wire					PHY_INT, PHY_INT_;
	wire					PHY_MDC;
	wire					PHY_MDIO;
	wire					PHY_RESET, PHY_RESET_;
	
	wire					PHY_TX_GCLK;
	wire					PHY_TX_CLK;
	wire					PHY_TX_EN, PHY_TX_ER;
	wire	[EWidth-1:0]	PHY_TX_D;
			
	wire					PHY_RX_CLK, PHY_RX_DV, PHY_RX_ER;
	wire	[EWidth-1:0]	PHY_RX_D;	
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	CUT
	//--------------------------------------------------------------------------
	FPGA_TOP_ML505 	#(			.UARTBaud(			UARTBaud), 
								.UARTDataBits(		UARTDataBits),
								.UARTParity(		UARTParity),
								.UARTStopBits(		UARTStopBits))
					CUT(		/*
									Declare Clock, Reset, Serial, etc 
									connections as you need them
								*/
								.PHY_COL(			PHY_COL),
								.PHY_CRS(			PHY_CRS),
								.PHY_INT_(			PHY_INT_),
								.PHY_MDC(			PHY_MDC),
								.PHY_MDIO(			PHY_MDIO),
								.PHY_RESET_(		PHY_RESET_),
								.PHY_RXCLK(			PHY_RX_CLK),
								.PHY_RXCTL_RXDV(	PHY_RX_DV),
								.PHY_RXER(			PHY_RX_ER),
								.PHY_RXD(			PHY_RX_D),
								.PHY_TXC_GTXCLK(	/* Unconnected */),
								.PHY_TXCLK(			PHY_TX_CLK),
								.PHY_TXCTL_TXEN(	PHY_TX_EN),
								.PHY_TXER(			PHY_TX_ER),
								.PHY_TXD(			PHY_TX_D));					
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Ethernet PHY Model
	//--------------------------------------------------------------------------
	assign	PHY_RESET =								~PHY_RESET_;
	assign	PHY_INT =								~PHY_INT_;
	
	EthernetPHYModel #(			.ClockFreq(			ClockFreq),
								.EthAddress(		5'd07),
								.EthAutoEnable(		0),
								.EthManSpeed(		2'b01), // 100 Mb/s
								.EthManFullDuplex(	1),
								.EthLoopback(		0),
								.EthIWidth(			8),
								.EthPHY(			0), // Marvell Alaska PHY on the ML505/XUPv5
								.EthNPorts(			1),
								.LoopDelay(			LoopDelay),
								.TXClockDelay(		TXClockDelay),
								.RXClockDelay(		RXClockDelay))
					EModel(		.PHY_COL(			PHY_COL),
								.PHY_CRS(			PHY_CRS),
								.PHY_INT(			PHY_INT),
								.PHY_MDC(			/* Unconnected */),
								.PHY_MDIO(			/* Unconnected */),
								.PHY_RESET(			PHY_RESET),
								.Speed(				/* Unconnected */),
								.LinkUp(			1'b1),
								.PHY_TX_GCLK(		/* Unconnected */),
								.PHY_TX_CLK(		PHY_TX_CLK),
								.PHY_TX_EN(			PHY_TX_EN),
								.PHY_TX_ER(			PHY_TX_ER),
								.PHY_TX_D(			PHY_TX_D),
								.PHY_RX_CLK(		PHY_RX_CLK),
								.PHY_RX_DV(			PHY_RX_DV),
								.PHY_RX_ER(			PHY_RX_ER),
								.PHY_RX_D(			PHY_RX_D));											
	//--------------------------------------------------------------------------