//==============================================================================
//	File:		$URL: svn+ssh://repositorypub@repository.eecs.berkeley.edu/public/Projects/GateLib/trunk/Firmware/Ethernet/Hardware/PHY/Test/EthernetPHYModel.v $
//	Version:	$Revision: 27062 $
//	Author:		Greg Gibeling (http://www.gdgib.com/)
//	Copyright:	Copyright 2003-2010 UC Berkeley
//==============================================================================

//==============================================================================
//	Section:	License
//==============================================================================
//	Copyright (c) 2005-2010, Regents of the University of California
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification,
//	are permitted provided that the following conditions are met:
//
//		- Redistributions of source code must retain the above copyright notice,
//			this list of conditions and the following disclaimer.
//		- Redistributions in binary form must reproduce the above copyright
//			notice, this list of conditions and the following disclaimer
//			in the documentation and/or other materials provided with the
//			distribution.
//		- Neither the name of the University of California, Berkeley nor the
//			names of its contributors may be used to endorse or promote
//			products derived from this software without specific prior
//			written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//==============================================================================

//==============================================================================
//	Section:	Defines and Constants
//==============================================================================
`timescale		1 ns/1 ps
//==============================================================================

//------------------------------------------------------------------------------
//	Module:		EthernetPHYModel
//	Desc:		A simulation model of an Ethernet PHY device.
//	Author:		<a href="http://www.gdgib.com/">Greg Gibeling</a>
//	Version:	$Revision: 27062 $
//------------------------------------------------------------------------------
module	EthernetPHYModel(
			//------------------------------------------------------------------
			//	Ethernet Control
			//------------------------------------------------------------------
			PHY_COL,
			PHY_CRS,
			PHY_INT,
			PHY_MDC,
			PHY_MDIO,
			PHY_RESET,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	Status Inputs
			//------------------------------------------------------------------
			Speed,
			LinkUp,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	Transmit Data Input
			//------------------------------------------------------------------
			PHY_TX_GCLK,
			PHY_TX_CLK,
			PHY_TX_EN,
			PHY_TX_ER,
			PHY_TX_D,
			//------------------------------------------------------------------
			
			//------------------------------------------------------------------
			//	Receive Data Output
			//------------------------------------------------------------------
			PHY_RX_CLK,
			PHY_RX_DV,
			PHY_RX_ER,
			PHY_RX_D
			//------------------------------------------------------------------
		);
	//--------------------------------------------------------------------------
	//	Parameters
	//--------------------------------------------------------------------------
	`include "EthernetPHYParams.constants"
	parameter				LoopDelay =				1,
							TXClockDelay =			2,
							RXClockDelay =			3,
							Debug =					1;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	State Encodings
	//--------------------------------------------------------------------------
	localparam				STATE_Width =			2,
							STATE_Command =			2'h0,
							STATE_Read =			2'h1,
							STATE_Write =			2'h2,
							STATE_Ignore =			2'h3;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Ethernet Control
	//--------------------------------------------------------------------------
	output					PHY_COL, PHY_CRS;
	output reg				PHY_INT =				0;
	input					PHY_MDC;
	inout					PHY_MDIO;
	input					PHY_RESET;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Status Inputs
	//--------------------------------------------------------------------------
	input	[EthSWidth-1:0]	Speed;
	input	[ETHLNK_Width-1:0] LinkUp;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Transmit Data Input
	//--------------------------------------------------------------------------
	input					PHY_TX_GCLK;
	output					PHY_TX_CLK;
	input					PHY_TX_EN, PHY_TX_ER;
	input	[EthEWidth-1:0]	PHY_TX_D;
	//--------------------------------------------------------------------------
			
	//--------------------------------------------------------------------------
	//	Receive Data Output
	//--------------------------------------------------------------------------
	output					PHY_RX_CLK, PHY_RX_DV, PHY_RX_ER;
	output	[EthEWidth-1:0]	PHY_RX_D;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Wires & Regs
	//--------------------------------------------------------------------------
	wire					ClockF25, ClockF2_5;
	reg						MuxedClock;
	
	reg		[STATE_Width-1:0] CurrentState, NextState;
	reg		[5:0]			BitCount =				0;
	reg		[47:0]			BitsIn;
	reg		[15:0]			BitsOut =				{16{1'bz}};
	reg		[15:0]			MDIRegs[31:0];
	
	wire	[ETHSPEED_Width-1:0] WideSpeed;
	wire					SpeedIntEnable, LinkUpIntEnable;
	reg		[EthSWidth-1:0]	OldSpeed;
	reg		[ETHLNK_Width-1:0] OldLinkUp;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Assigns
	//--------------------------------------------------------------------------
	assign	#(TXClockDelay) PHY_TX_CLK =			MuxedClock;
	assign	#(RXClockDelay) PHY_RX_CLK =			MuxedClock;
	assign	#(LoopDelay) PHY_RX_DV =				LinkUp[0] ? PHY_TX_EN : 1'b0;
	assign	#(LoopDelay) PHY_RX_ER =				LinkUp[0] ? PHY_TX_ER : 1'b0;
	assign	#(LoopDelay) PHY_RX_D =					LinkUp[0] ? PHY_TX_D : {EthEWidth{1'bz}};
	
	assign	PHY_MDIO =								BitsOut[15];
	
	generate if (!EthAutoEnable) begin:SPEEDMAN
		assign	WideSpeed =							EthManSpeed;
	end else if (EthSWidth < ETHSPEED_Width) begin:SPEEDWIDEN
		assign	WideSpeed =							{{ETHSPEED_Width-EthSWidth{1'b0}}, Speed};
	end else begin:SPEEDNARROW
		assign	WideSpeed =							Speed[1:0];
	end endgenerate
	assign	SpeedIntEnable =						(EthAutoEnable ? 1'b1 : 1'b0) & ((EthPHY == ETHPHY_Intel_LXT974) ? (MDIRegs[5'h11][1] & MDIRegs[5'h13][12]) : ((EthPHY == ETHPHY_Marvell_88E1111) ? (MDIRegs[5'h12][14] | MDIRegs[5'h12][11]) : (MDIRegs[5'h1b][2] | ~MDIRegs[5'h10][12])));
	assign	LinkUpIntEnable =						((EthPHY == ETHPHY_Intel_LXT974) ? (MDIRegs[5'h11][1] & MDIRegs[5'h13][12]) : ((EthPHY == ETHPHY_Marvell_88E1111) ? (MDIRegs[5'h12][11] | MDIRegs[5'h12][10]) : (MDIRegs[5'h1b][1] | ~MDIRegs[5'h10][12])));
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Clock Source
	//--------------------------------------------------------------------------
	ClockSource #(25000000) ClockGenF25(.Enable(~PHY_RESET), .Clock(ClockF25));
	ClockSource #(2500000) ClockGenF2_5(.Enable(~PHY_RESET), .Clock(ClockF2_5));
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Clock Mux
	//--------------------------------------------------------------------------
	always @ (*) begin
		case (WideSpeed)
			ETHSPEED_10Mb: MuxedClock =				ClockF2_5;
			ETHSPEED_100Mb: MuxedClock =			ClockF25;
			ETHSPEED_1Gb: MuxedClock =				PHY_TX_GCLK;
			default: MuxedClock =					1'bx;
		endcase
	end
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	MDI Initial Setup
	//--------------------------------------------------------------------------
	initial begin
		CurrentState =								STATE_Command;
		
		MDIRegs[0] =								16'b0111_0001_1000_0000;
		MDIRegs[1] =								{13'b1111_1110_0010_1, LinkUp[0], 2'b01};
		if (EthPHY == ETHPHY_Marvell_88E1111) MDIRegs[5'h11] = {WideSpeed[1:0], 3'b11_1, LinkUp[0], 10'b00_0000_0000};
		else if (EthPHY == ETHPHY_Intel_LXT974) MDIRegs[5'h14] = {2'b00, LinkUp[0], 1'b1, WideSpeed[0], 11'b011_0000_0000};
		else if (EthPHY == ETHPHY_Broadcom_BCM5481) MDIRegs[5'h11] = {5'b0000_0, WideSpeed[1:0], LinkUp[0], 8'b0000_0000};
	end
	
	always @ (posedge PHY_RESET or posedge PHY_MDC) begin
		if (PHY_RESET) begin
			CurrentState =							STATE_Command;
			
			MDIRegs[0] =							16'b0111_0001_1000_0000;
			MDIRegs[1] =							{13'b1111_1110_0010_1, LinkUp[0], 2'b01};
			if (EthPHY == ETHPHY_Marvell_88E1111) MDIRegs[5'h11] = {WideSpeed[1:0], 3'b11_1, LinkUp[0], 10'b00_0000_0000};
			else if (EthPHY == ETHPHY_Intel_LXT974) MDIRegs[5'h14] = {2'b00, LinkUp[0], 1'b1, WideSpeed[0], 11'b011_0000_0000};
			else if (EthPHY == ETHPHY_Broadcom_BCM5481) MDIRegs[5'h11] = {5'b0000_0, WideSpeed[1:0], LinkUp[0], 8'b0000_0000};
		end
	end
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	MDI Current State Register
	//--------------------------------------------------------------------------
	always @ (posedge PHY_MDC) begin
		CurrentState <=								NextState;
		BitCount <=									BitCount + 1;
		BitsIn <=									{BitsIn[46:0], PHY_MDIO};
		
		if ((CurrentState == STATE_Command) && (NextState == STATE_Read)) begin
			BitsOut <=								MDIRegs[BitsIn[5:1]];
			if (Debug) $display("INFO[%m @ %t]: Ethernet Model(%x) READ MDI[%x] = 0x%x", $time, EthAddress, BitsIn[5:1], MDIRegs[BitsIn[5:1]]);
		end else BitsOut <=							{BitsOut[14:0], 1'bz};
	end
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	MDI Next State Logic
	//--------------------------------------------------------------------------
	always @ (*) begin
		NextState =									CurrentState;
		
		case (CurrentState)
			STATE_Command: if (BitCount == 47) begin
				if (~&BitsIn[46:15]) $display("ERROR[%m @ %t]: MDI preamble wasn't all 1s!", $time);
				if (BitsIn[14:13] != 2'b01) $display("ERROR[%m @ %t]: MDI startframe was wrong!", $time);
				if (~^BitsIn[12:11]) $display("ERROR[%m @ %t]: MDI operation was neither a read nor a write!", $time);
				
				if (BitsIn[10:6] != EthAddress) NextState = STATE_Ignore;
				else if (BitsIn[12:11] == 2'b10) NextState = STATE_Read;
				else if (BitsIn[12:11] == 2'b01) NextState = STATE_Write;
				else NextState =					STATE_Ignore;
			end
			STATE_Read: if (BitCount == 63) NextState = STATE_Command;
			STATE_Write: if (BitCount == 63) begin
				MDIRegs[BitsIn[21:17]] =			{BitsIn[14:0], PHY_MDIO};
				if (Debug) $display("INFO[%m @ %t]: Ethernet Model(%x) WRITE MDI[%x] = 0x%x", $time, EthAddress, BitsIn[21:17], {BitsIn[14:0], PHY_MDIO});
				NextState =							STATE_Command;
			end
			STATE_Ignore: if (BitCount == 63) NextState = STATE_Command;
		endcase
		
		MDIRegs[1] =								{13'b1111_1110_0010_1, LinkUp[0], 2'b01};
		if (EthPHY == ETHPHY_Marvell_88E1111) MDIRegs[5'h11] = {WideSpeed[1:0], 3'b11_1, LinkUp[0], 10'b00_0000_0000};
		else if (EthPHY == ETHPHY_Intel_LXT974) MDIRegs[5'h14] = {2'b00, LinkUp[0], 1'b1, WideSpeed[0], 11'b011_0000_0000};
		else if (EthPHY == ETHPHY_Broadcom_BCM5481) MDIRegs[5'h11] = {5'b0000_0, WideSpeed[1:0], LinkUp[0], 8'b0000_0000};
	end
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Interrupt Generation
	//--------------------------------------------------------------------------
	always @ (*) begin
		while ((OldSpeed !== Speed) || (OldLinkUp !== LinkUp)) begin
			if (((OldSpeed !== Speed) && SpeedIntEnable) || ((OldLinkUp !== LinkUp) && LinkUpIntEnable)) begin
				OldSpeed =							Speed;
				OldLinkUp =							LinkUp;
				
				#(100)
				PHY_INT =							1'b1;
				#(300)
				PHY_INT =							1'b0;
			end else begin
				OldSpeed =							Speed;
				OldLinkUp =							LinkUp;
			end
		end
	end
	//--------------------------------------------------------------------------
endmodule
//------------------------------------------------------------------------------
